/* 
 * camera.h -- Vectors and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.9.0rc1
 * Last Change: January 28, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * In geometry, a "frame" is a right-handed orthonormal basis, that is,
 * an ordered set of three mututally perpendicular unit vectors, oriented
 * according to the right-hand rule. A frame has nothing to do with
 * picture frames.
 *
 * ePiX uses a camera metaphor to view points in R^3. The camera consists
 * of:
 *   - a frame ({sea, sky, eye}, with the intuitive meanings:)
 *   - a point to look at (the "target")
 *   - the distance from the camera to the target
 *   - a projection mapping from R^3 to R^2 (the "lens")
 *
 * While specific behavior is determined by the lens, the idea is that the
 * viewer sits at given <distance> from the <target>, in the direction of
 * the <eye>. The vectors <sea> and <sky> point horizontally to the right
 * and vertically upward on the screen. A <distance> of zero is taken to
 * mean "infinitely far away"; for example, point projection (<shadow>)
 * becomes orthogonal projection along the <eye> axis. Again, a specific
 * lens can override this behavior, for example by ignoring the distance.
 *
 *
 * This file provides:
 *   - the frame class, and routines for rotating a frame about the axes
 *     determined by its elements. (Note that there are no methods for
 *     rotating about fixed coordinate axes, and that rotations in R^3 do
 *     not generally commute.
 *
 *   - the camera class and routines:
 *       void rotate_sea(double); // change camera orientation
 *       void rotate_sky(double); // about respective axes
 *       void rotate_eye(double);
 *
 *       void range(double d); // set distance from camera to origin
 *       void look_at(P arg); // set target (center of field of view)
 *       void lens(pair (*screen_projection(P))); // set projection mapping
 *
 *       pair shoot(P arg); // project a point to the screen ("take a photo")
 *
 */

#ifndef _EPIX3d_CAMERA
#define _EPIX3d_CAMERA

#include "vectors.h"
#include "points.h"
#include "pairs.h"
#include "functions.h"

namespace ePiX3d {

  // standard Cartesian frame at origin
  const V E_1 = V(1,0,0);
  const V E_2 = V(0,1,0);
  const V E_3 = V(0,0,1);

  class frame {
  private:
    // orthonormal triple
    V frame1;
    V frame2;
    V frame3;

  public:
    //constructor
    frame(void) { frame1 = E_1; frame2 = E_2; frame3 = E_3; }

    // frame elements
    V sea() { return frame1; }
    V sky() { return frame2; }
    V eye() { return frame3; }

    // rotations about frame elements
    frame rotate_frame1(double angle)
      {
	V temp2 = frame2;
	V temp3 = frame3;

	frame2 = 
	  (ePiX3d::cos(angle)*(temp2)) - (ePiX3d::sin(angle)*(temp3));
	frame3 = 
	  (ePiX3d::sin(angle)*(temp2)) + (ePiX3d::cos(angle)*(temp3));

	return *this;
      }


    frame rotate_frame2(double angle)
      {
	V temp3 = frame3;
	V temp1 = frame1;

	frame3 = 
	  (ePiX3d::cos(angle)*(temp3)) - (ePiX3d::sin(angle)*(temp1));

	frame1 = 
	  (ePiX3d::sin(angle)*(temp3)) + (ePiX3d::cos(angle)*(temp1));

	return *this;
      }

    frame rotate_frame3(double angle)
      {
	V temp1 = frame1;
	V temp2 = frame2;

	frame1 = 
	  (ePiX3d::cos(angle)*(temp1)) - (ePiX3d::sin(angle)*(temp2));

	frame2 = 
	  (ePiX3d::sin(angle)*(temp1)) + (ePiX3d::cos(angle)*(temp2));

	return *this;
      }

  }; // end of class frame

  class epix3d_camera {
  private:
    frame orientation;
    double distance;
    P target;
    pair (*screen_projection)(P);

  public:

    epix3d_camera(void) 
      { 
	screen_projection = shadow; // orthog projection when distance = 0
      }

    double get_range(void) { return distance; }
    P get_target(void) { return target; }

    // pitch: rotate camera up/down
    void rotate_sea(double angle)
      { 
	orientation = orientation.rotate_frame1(angle);
      }

    // yaw: rotate camera left/right
    void rotate_sky(double angle)
      { 
	orientation = orientation.rotate_frame2(angle);
      }

    // roll: rotate camera about viewing axis
    void rotate_eye(double angle)
      { 
	orientation = orientation.rotate_frame3(angle);
      }

    // get/set distance from camera to origin
    void range(double d) { distance = d; }

    // set target (center of field of view)
    void look_at(P arg) { target = arg; }

    // set projection mapping
    void lens(pair proj(P)) { screen_projection = proj; }

    // project a point to the screen ("shoot a photo")
    pair shoot(P arg) { return screen_projection(arg); }

    // screen projection mappings ("lenses")
    friend pair shadow(P arg);
    friend pair fisheye(P arg);

  }; // end of class epix3d_camera 

} /* end of namespace */

#endif /* _EPIX3d_CAMERA */
