/* 
 * labels.h -- LaTeX boxes that have a well-defined position
 *
 * This file is part of ePiX, a preprocessor for creating high-quality
 * line figures in LaTeX
 *
 * Version 0.9.0rc1
 * Last Change: February 04, 2003
 */
 
/*
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */
 
/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _EPIX3d_LABELS
#define _EPIX3d_LABELS

#include <iostream>
#include <string>

#include "points.h"
#include "pairs.h"
// #include "functions.h"
#include "lengths.h"
// #include "camera.h"
#include "style.h"

namespace ePiX3d {

  enum epix_mark_type {NULL_MARK, PATH, CIRC, SPOT, RING, DOT, DDOT, 
		       PLUS, OPLUS, TIMES, OTIMES, DIAMOND, UP, DOWN, 
		       BOX, BBOX, H_TICK, V_TICK};

  // 0 = no alignment
  enum epix_label_posn {no_align, c, r, tr, rt, t, tl, lt, l, bl, lb, b, br, rb};

  /*
   * label -- put string constant <label_text> 
   * at Cartesian position <pair> offset by <vect> true points.
   * Accepts an optional LaTeX-style positioning argument after the offset.
   * If no offset is specified, the label is centered at the given Cartesian
   * location.
   * masklabel requires the "color" package, and places the text in 
   * a white box, to mask what is underneath.
   */

  extern epix3d_style current;

  class label {

  private:
    P base;
    pair offset;

    epix_label_posn alignment;
    color_data color;
    bool masked;

    std::string label_text;

  public:
    // void constructor
    label(void)
      {
	base = P();
	offset = t2p(pair());
	
	alignment = no_align;
	color = current.color;
	masked = false;

	label_text = "";
      }

    // unmasked constructors
    label(P here, pair shift, std::string my_label)
      {
	base = here;
	offset = t2p(shift); // convert true pt to screen coords

	alignment = no_align;
	color = current.color;
	masked = false;

	label_text = my_label;
      }

    label(P here, std::string my_label)
      {
	base = here;
	offset = pair(0,0);
      
	alignment = no_align;
	color = current.color;
	masked = false;
	
	label_text = my_label;
      }
    
    label(P here, pair shift, std::string my_label,
	  epix_label_posn my_align)
      {
	base = here;
	offset = t2p(shift); // convert true pt to screen coords

	alignment = my_align;
	color = current.color;
	masked = false;

	label_text = my_label;
      }
    /*
      // unmasked constructors
      label(P here, pair shift, std::string my_label);
      label(P here, std::string my_label);
      label(P here, pair shift, std::string my_label,
      epix_label_posn my_align);
    */

    // masked constructors
    friend label masklabel(P here, pair shift, std::string my_label);
    friend label masklabel(P here, std::string my_label);
    friend label masklabel(P here, pair shift, std::string my_label,
			   epix_label_posn my_align);

    friend label marker(P here, epix_mark_type type);

    friend label ring(P, double);

    // Empty and filled LaTeX circles
    inline friend label circ(P here) { return marker(here, CIRC); }
    inline friend label spot(P here) { return marker(here, SPOT); }
    inline friend label  dot(P here) { return marker(here,  DOT); }
    inline friend label ddot(P here) { return marker(here, DDOT); }

    //    inline friend label dot(P here, double);

    friend void print(label arg);
    friend void print(label* arg);
  }; // end of class

} /* end of namespace */

#endif /* _EPIX3d_LABELS */
