/*
 * lengths.h -- ePiX true and LaTeX length manipulation
 *
 * Andrew D. Hwang            <ahwang@mathcs.holycross.edu>
 *
 * Version 0.9.0rc1
 * Last Change: February 4, 2003
 *
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * In ePiX, there are five systems of coordinates:               *
 *                                                               *
 *   User (*)   -- All the user needs to know about              *
 *   Cartesian  -- Abstract internal coordinates (3-dim)         *
 *   Screen (*) -- Abstract internal coordinates (2-dim)         *
 *   Picture    -- The language of the LaTeX front end           *
 *   True       -- Physical location on page from lower left     *
 *                                                               *
 *   (*) New in ePiX 0.9.x                                       *
 *                                                               *
 * This file handles the conversion from screen to picture       *
 * coordinates.                                                  *
 *                                                               *
 * User and Cartesian coordinates are both 3-dimensional, while  *
 * Picture and True coordinates are 2-dimensional. Usually User  *
 * and Cartesian coordinates coincide, but they need not; for    *
 * example, User coordinates might be spherical or cylindrical.  *
 *                                                               *
 * The <P>-valued mapping ePiX::coordinate_system(<P>) handles   *
 * the transformation of User to Cartesian coordinates. This     *
 * mapping is the identity by default. Points' coordinates are   *
 * always stored internally in Cartesian, so that the coordinate *
 * system need not be stored with plot data structures.          *
 *                                                               *
 * ePiX reads the entire input file and generates data structures*
 * that represent the objects to be drawn, then determines where *
 * on the page each object goes.
                               
 * Recognized LaTeX units are: cm, in, mm, pc, and pt            *
 *                                                               *
 * All objects and locations are specified in C by the user; the *
 * conversion to true coordinates is transparent. The rationale  *
 * for retaining LaTeX coordinates, rather than forcing all ePiX *
 * figures to be specified in true pt, is of course that figures *
 * are sized according to printing requirements, and often not   *
 * in terms of true pt. The reason for using true pt is that raw *
 * scaling of a LaTeX figure is not robust; label positions must *
 * be specified with true pt offsets if their locations are to   *
 * scale properly when a figure is resized.                      *
 *                                                               *
 * An ePiX figure is a rectangle, described as follows:          *
 *   Cartesian: [x_min, x_max] x [y_min, y_max]                  *
 *   Picture:   [0, h_size] x [0, v_size] + (h_offset, v_offset) *
 *   True:      Only (La)TeX knows/needs to know this            *
 *                                                               *
 * ePiX provides six scale-changing functions:                   *
 *   c2p, p2c -- Cartesian -> LaTeX and back for points          *
 *   c2s, s2c -- Cartesian -> LaTeX and back for vectors         *
 *   p2t, t2p -- LaTeX -> True and back                          *
 *                                                               *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _EPIX3d_LENGTHS
#define _EPIX3d_LENGTHS

#include <string>

#include "points.h"
#include "vectors.h"
#include "pairs.h"

namespace ePiX3d {

  /* If less than EPIX_EPSILON, set to zero */
  double truncate(double);
  pair truncate(pair);

  /* Functions for declaring variables */
  void bounding_box(P, P);
  void picture(P);
  void unitlength (const char *);
  void offset(P);

  /* Coordinate conversion routines */
  pair c2p(pair);
  pair p2c(pair);

  pair c2s(pair);
  pair s2c(pair);
  
  double p2t(double);
  double t2p(double);
  pair p2t(pair);
  pair t2p(pair);

  double raw_len(pair); // Pythagorean length of pair/triple
  double raw_len(V p);

  /* Length of pair given in picture coords */
  double true_len(pair); // length in true pt

  pair normalize(pair); // parallel true pt unit vector

} /* end of namespace */

#endif /* _EPIX3d_LENGTHS */
