/* 
 * output.cc -- output file formatting routines
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.9.0rc1
 * Last Change: February 07, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <iostream>
#include <string>
#include <cstdio>

#define _EPIX_VERSION // *epix_version accessible from this file only

#include "style.h"
#include "points.h"
#include "pairs.h"
#include "lengths.h"
#include "output.h"
#include "camera.h"

namespace ePiX3d {

  using std::cout;
  using std::cerr;

  extern const char *epix_version;

  extern epix3d_style current;
  
  double epix_stretch_factor=1.0;

  double h_size, v_size, h_offset, v_offset;
  double x_min, x_max, y_min, y_max, z_min, z_max;
  double x_size, y_size, z_size;
  double pic_size;
  char *pic_unit;

  int epix_begin_count=0; // number of calls to begin()

  void epix_error(std::string error)
  {
    cerr << "ePiX ERROR: " << error << "\n";
    exit(1);
  }

  void epix_warning(std::string warning)
  {
    cerr << "ePiX WARNING: " << warning << "\n";
  }

  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
   *                 Sectioning Functions                  *
   \* * * * * * * * * * * * * * * * * * * * * * * * * * * * */

  /* Print standard ePiX picture header */
  void begin(void)
  {
    if (epix_begin_count == 0)
      std::fprintf (stdout, "%% ePiX-%s\n", epix_version);
    ++epix_begin_count;

    std::fprintf (stdout, "%%");
    std::fprintf (stdout, "\n%%   Cartesian bounding box: ");
    if ( z_min != z_max )
      std::fprintf (stdout, "[%g,%g] x [%g,%g] x [%g,%g]",
		    x_min, x_max, y_min, y_max, z_min, z_max);
    else 
      std::fprintf (stdout, "[%g,%g] x [%g,%g]", x_min, x_max, y_min, y_max);

    std::fprintf (stdout, "\n%%   Actual size: %g%s x %g%s",
		  h_size*pic_size, pic_unit, v_size*pic_size, pic_unit);
    std::fprintf (stdout, "\n%%   Figure offset: %s by %g%s, %s by %g%s",
	     (h_offset >= 0) ? "right" : "left", 
	     fabs(h_offset)*pic_size, pic_unit, 
	     (v_offset >= 0) ? "up"    : "down", 
	     fabs(v_offset)*pic_size, pic_unit);
    std::fprintf (stdout, "\n%%");
    std::fprintf (stdout, "\n\\setlength{\\unitlength}{%g%s}", 
		  pic_size, pic_unit);
    std::fprintf (stdout, "\n\\begin{picture}(%g,%g)(%g,%g)", \
		  h_size, v_size, -h_offset, -v_offset);
    end_stanza();
    
  }

  void end(void)  // needs real definition
  {
    cout << "\n\\end{picture}\n";
  }

  // output methods
  // print Cartesian pair in LaTeX coordinates
  void print(pair point)
  {
    pair out = c2p(point);
    cout << out;
  }

  // print pair without conversion
  void raw_print(pair point)
  {
    cout << truncate(point);
  }

  extern epix3d_camera camera;
  // print P
  void print(P point)
  {
    pair out = c2p(camera.shoot(point));
    cout << out;
  }

void start_path(PATH_STYLE path_style, double stretch)
{
  cout << "\n";

  switch(path_style) 
    {

    case SOLID:
      cout << "\\path";
      break;

    case DASHED:
      cout.precision(3);
      cout << "\\dashline{" << t2p(epix_stretch_factor*EPIX_DASH_LENGTH) <<"}";
      break;

    case DOTTED:
      cout.precision(3);
      cout << "\\dottedline{" << t2p(epix_stretch_factor*EPIX_DOT_SEP) << "}";
      break;

    default:
      cout << "\\path";
      break;
    }
}

  /* Unified comment function */
  void epix_comment(std::string type)
  {
    cout << "\n%% " << type << ":\n%%";
  } 

  /* End current stanza of .eepic file */
  void end_stanza(void)
  {
    cout.fill('-');
    cout << "\n%%\n%%";
    cout.width(EPIX_FILE_WIDTH-4);
    cout << "%%";
  }

} /* end of namespace */
