/* 
 * points.h -- Points and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.9.0rc1
 * Last Change: January 31, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * A <P> ("point") is conceptually a LOCATION in 3-dimensional space.
 * This file provides constructors, coordinate functions, projection
 * operators to the coordinate axes/planes, and output. Most of these
 * functions are inlined.
 *
 * It is not meaningful to perform algebraic operations (addition, dot
 * product, etc.) on points; the <V> (vector) type is provided for
 * this purpose. It *is* meaningful to take the difference of two
 * points, to obtain a vector (displacement). This operation is
 * defined in vectors.*, not in this file.  
 *
 * Points are represented as elements of 3-dimensional projective
 * space, that is, with a fourth coordinate which is normally 1. This
 * scheme allows parallel segments to have a point of intersection "at
 * infinity": Two segments parallel to (a,b,c) intersect at (0,a,b,c).
 */

#ifndef _EPIX3d_POINTS
#define _EPIX3d_POINTS

#include <cstdlib>

namespace ePiX3d {

  // points in Cartesian coordinates
  class P {

  private:
    // coordinates
    double X1, X2, X3;

  public:  
    // constructors
    P(void) { X1 = X2 = X3 = 0; }
    P(double x1, double x2) { X1 = x1; X2 = x2; X3 = 0; }
    P(double x1, double x2, double x3) { X1 = x1; X2 = x2; X3 = x3; }

    // coordinate functions
    double x1() { return X1; }
    double x2() { return X2; }
    double x3() { return X3; }

    //  projection to coordinate axes
    friend P pr1(P arg) { return P(arg.X1, 0, 0); }
    friend P pr2(P arg) { return P(0, arg.X2, 0); }
    friend P pr3(P arg) { return P(0, 0, arg.X3); }

    // to coordinate planes
    friend P pr12(P arg) { return P(arg.X1, arg.X2, 0); }
    friend P pr13(P arg) { return P(arg.X1, 0, arg.X3); }
    friend P pr23(P arg) { return P(0, arg.X2, arg.X3); }

  }; // end of class P
  
  // alternate notation for coordinates
  inline double x1(P arg) { return arg.x1(); }
  inline double x2(P arg) { return arg.x2(); }
  inline double x3(P arg) { return arg.x3(); }

  // (in)equality
  bool operator == (P arg1, P arg2);
  bool operator != (P arg1, P arg2);

  bool is_finite(P arg);
  
  // convex linear combinations (t defaults to 0.5)
  P midpoint(P arg1, P arg2, double t);

} /* end of namespace ePiX3d */

#endif /* _EPIX_POINTS */
