/* 
 * vectors.cc -- Vectors and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.9.0rc1
 * Last Change: January 28, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "vectors.h"
#include "points.h"
#include "style.h"
#include "functions.h"
// #include "lengths.h"

namespace ePiX3d {

  // head and tail
  //  P tail(V arg) { return arg.tail(); }
  P head(V arg)
  {
    double u1 = arg.tail().x1() + arg.x1();
    double u2 = arg.tail().x2() + arg.x2();
    double u3 = arg.tail().x3() + arg.x3();

    return P(u1, u2, u3);
  }

  // ePiX3d::trig functions are sensitive to angular units

  // polar constructors; n.b. ePiX3d::trig functions
  V polar(double r, double t) { return V(r*ePiX3d::cos(t), r*ePiX3d::sin(t)); }
  V cis(double t) { return V(ePiX3d::cos(t), ePiX3d::sin(t)); }

  V polar(P tail, double r, double t) 
  { 
    return V(tail, r*ePiX3d::cos(t), r*ePiX3d::sin(t)); 
  }
  V cis(P tail, double t) 
  { 
    return V(tail, ePiX3d::cos(t), ePiX3d::sin(t)); 
  }

  // vector space operations; error if vectors not based at same point
  V operator+ (V arg1, const V arg2) { return arg1 += arg2; }
  V operator- (V arg1, const V arg2) { return arg1 -= arg2; }
  V operator- (V arg) { return arg *= -1; }

  // cross product
  V operator* (V arg1, const V arg2) { return arg1 *= arg2; }

  // scalar multiplication
  V operator* (const double c, V v) { return v *= c; }

  // dot product and norm
  double operator| (const V arg1, const V arg2) 
  { 
    return (arg1.X1)*(arg2.X1) + (arg1.X2)*(arg2.X2) + (arg1.X3)*(arg2.X3);
  }

  double norm(const V arg) { return sqrt(arg|arg); }

  // componentwise product
  V operator& (V arg1, const V arg2) { return arg1 &= arg2; }

  // orthogonalization: subtract arg2 component from arg1
  V operator/ (V arg1, const V arg2) { return arg1 /= arg2; }

  // operators involving points
  P operator+ (P arg1, V arg2)
  {
    if ( arg2.tail() != arg1 )
      epix_error("Adding point to vector based elsewhere");

    // else
    return P(arg1.x1() + arg2.x1(), 
	     arg1.x2() + arg2.x2(), 
	     arg1.x3() + arg2.x3());
  }

  P operator+ (V arg1, P arg2)
  {
    if ( arg1.tail() != arg2 )
      epix_error("Adding point to vector based elsewhere");

    // else
    return P(arg1.x1() + arg2.x1(), 
	     arg1.x2() + arg2.x2(), 
	     arg1.x3() + arg2.x3());
  }

  V operator- (P arg1, P arg2)
  {
    return V(arg2,
	     arg1.x1() - arg2.x1(), 
	     arg1.x2() - arg2.x2(), 
	     arg1.x3() - arg2.x3());
  }

  // (in)equality
  bool operator == (V arg1, V arg2)
  {
    return ((arg1.x1() == arg2.x1()) && 
	    (arg1.x2() == arg2.x2()) && 
	    (arg1.x3() == arg2.x3()) &&
	    (arg1.tail() == arg2.tail()));
  }
 
  bool operator != (V arg1, V arg2)
  {
    return ((arg1.x1() != arg2.x1()) || 
	    (arg1.x2() != arg2.x2()) || 
	    (arg1.x3() != arg2.x3()) ||
	    (arg1.tail() != arg2.tail()));
  }

} /* end of namespace */

