/* 
 * vectors.h -- Vectors and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.9.0rc1
 * Last Change: January 27, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * A <V> ("vector") is conceptually a DISPLACEMENT from a point in 
 * 3-dimensional space. Vectors are represented as a basepoint and a
 * triple of numbers. ePiX works in non-Euclidean coordinate systems, 
 * and therefore must emphasize the distinction between locations
 * (points) and displacements (vectors). In coordinate systems other
 * than Cartesian, a basepoint *must* be specified when defining a
 * vector.
 *
 * Mathematically, a point (<P>) is an element of the manifold R^3, while
 * a vector (<V>) is an element of the tangent bundle, TR^3. Concretely, 
 * P pt = P(x,y,z) defines the point with coordinates (x,y,z) in the
 * current coordinate system, while V vec = V(pt, a, b, c) defines the
 * vector based at pt and having components (a,b,c) in the current
 * coordinate system. If the coordinate system is Cartesian and no basepoint
 * is specified, the origin is taken by default. This should result in the
 * expected behavior.
 *
 * This file provides:
 *   - constructors and algebraic operators for V, including vector
 *     space operations, dot product, the sum of a point and a vector,
 *     and the difference of points.
 *   - a <frame> class, a right-handed, orthonormal triple of V, and
 *     functions for rotating a frame about its elements.
 */

#ifndef _EPIX3d_VECTORS
#define _EPIX3d_VECTORS

#include <string>
#include <cmath>

#include "constants.h"
#include "points.h"
// #include "output.h"

namespace ePiX3d {

  void epix_error(std::string);

  class V {

  private:
    P basepoint;
    double X1, X2, X3;

  public:  
    // Cartesian constructors without basepoint
    V(void)
      { 
	basepoint = P(); // origin
	X1 = X2 = X3 = 0; // zero vector
      }

    V(double x1, double x2, double x3 = 0)
      { 
	basepoint = P(); 
	X1 = x1; 
	X2 = x2; 
	X3 = x3;
      }

    // Cartesian constructors with basepoint
    V(P tail)
      { 
	basepoint = tail; // Pt(x1(tail), x2(tail), x3(tail));
	X1 = X2 = X3 = 0;
      }

    V(P tail, double x1, double x2, double x3 = 0)
      { 
	basepoint = tail; 
	X1 = x1; 
	X2 = x2; 
	X3 = x3;
      }

    // component functions
    P& tail() { return basepoint; }
    double& x1() { return X1; }
    double& x2() { return X2; }
    double& x3() { return X3; }

    // move basepoint of vector to origin
    friend V carry_to_origin(V arg) { return V(arg.x1(), arg.x2(), arg.x3()); }

    // scalar multiplication
    V& operator*= (const double& c)
    {
      X1 *= c;
      X2 *= c;
      X3 *= c;

      return (*this); 
    }

    // increment operators
    V& operator+= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Sum of vectors based at different points");

      // else
      X1 += arg.X1;
      X2 += arg.X2;
      X3 += arg.X3;
    
      return (*this); 
    }

    V& operator-= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Difference of vectors based at different points");
  
      // else
      X1 -= arg.X1;
      X2 -= arg.X2;
      X3 -= arg.X3;

      return (*this); 
    }

    // cross product
    V& operator*= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Cross product of vectors based at different points");

      // else
      V temp = *this;

      X1 = (temp.X2)*(arg.X3) - (temp.X3)*(arg.X2);
      X2 = (temp.X3)*(arg.X1) - (temp.X1)*(arg.X3);
      X3 = (temp.X1)*(arg.X2) - (temp.X2)*(arg.X1);
      return (*this);
    }

    // dot product
    double operator|= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Dot product of vectors based at different points");

      // else
      double result;
      result += (X1*arg.X1);
      result += (X2*arg.X2);
      return result += (X3*arg.X3);
    }

    // componentwise product
    V& operator&= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Componentwise product of vectors based at different points");

      // else
      X1 *= arg.X1;
      X2 *= arg.X2;
      X3 *= arg.X3;

      return (*this); 
    }

    // orthogonalization
    V& operator/= (V arg)
    {
      if (tail() != arg.tail())
	epix_error("Orthogonalization of vectors based at different points");

      // else
      double norm = (arg.X1)*(arg.X1) + (arg.X2)*(arg.X2) + (arg.X3)*(arg.X3);
      if ( fabs(norm) < EPIX_EPSILON ) 
	return (*this);
      else 
	{
	  double c = (X1*arg.X1 + X2*arg.X2 + X3*arg.X3)/norm;
	  X1 -= c*arg.X1;
	  X2 -= c*arg.X1;
	  X3 -= c*arg.X1;
	  return *this;
	}
    }

    /*
      // cross product
      V& operator*= (V arg);
      // dot product
      double operator|= (V arg);
      // componentwise product
      V& operator&= (V arg);

      // orthogonalization
      V& operator/= (V arg);
    */

    friend P head(V arg);
    friend double operator| (const V arg1, const V arg2); // dot product

  }; // end of class V

  // alternate versions
  inline P tail(V arg) { return arg.tail(); }
  inline double x1(V arg) { return arg.x1(); }
  inline double x2(V arg) { return arg.x2(); }
  inline double x3(V arg) { return arg.x3(); }

  // vector space operators
  V operator+ (V arg1, const V arg2);
  V operator- (V arg1, const V arg2);
  V operator- (V arg); // unary minus
  V operator* (V arg1, const V arg2); // cross product
  V operator* (const double c, V arg); // scalar multiplication

  // miscellaneous operators
  V operator& (V arg1, const V arg2); // componentwise product
  V operator/ (V arg1, const V arg2); // orthogonalization
  double norm (V arg);  // norm

  // difference of points, sum of point and vector
  P operator+ (P arg1, V arg2);
  P operator+ (V arg1, P arg2);
  V operator- (P arg1, P arg2);

  // (in)equality
  bool operator == (V arg1, V arg2);
  bool operator != (V arg1, V arg2);

  V polar(double r, double t);
  V cis(double t);

  V polar(P tail, double r, double t);
  V cis(P tail, double t);

} /* end of namespace */

#endif /* _EPIX3d_VECTORS */
